/*
 *
 * eps2epsi.c - read an epsf file and convert it to epsi format
 *
 *	Runs on a SUN3, SUNOS 4.1. Not tested on any other platforms
 *
 *	Tested for the following applications that generate
 *	epsf files on the PC:
 *
 *		Corel Draw version 1.1
 *		Illustrator 88
 *
 *
 *	Note: This software requires the Tiff library software
 *	      written and maintained by Sam Leffler at UC Berkeley.
 *
 *		There is a mailing list associated with the software,
 *		tiff@ucbvax.berkeley.edu, that you can join by sending
 *		a message to tiff-request@ucbvax.berkeley.edu.
 *		You can also reach him directly at sam@ucbvax.berkeley.edu.
 *
 *
 * Copyright (C) 1990 by Russell Sutherland.
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose and without fee is hereby granted, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation.  This software is provided "as is" without express or
 * implied warranty.
 *
 */

#include <sys/file.h>
#include <sys/types.h>
#include <malloc.h>
#include <stdio.h>
#include <string.h>
#include "tiffio.h"

#define BYTE2LONG(x) ( *(x) | *((x)+1) << 8 | *((x)+2) << 16 | *((x)+3) << 24 )
#define BYTE2SHORT(x) ( *(x) | *((x)+1) << 8 )
#define STREQ(s1, s2)	( (strcmp((s1), (s2)) == 0) )
#define TIFF_TMP	"/tmp/tiff.out"
#define PS_TMP		"/tmp/ps.out"

FILE *fp, *fopen();
unsigned short tiff_width, tiff_height, tiff_photo;
unsigned short checksum;

off_t ps_offset, tiff_offset;
unsigned ps_length, tiff_length;
int mf_offset, mf_length;

main()
{
	process_header();
	gen_ps_file();
	gen_tiff_file();
	process_files();
/*
	cleanup_files();
*/
}


process_header()
{
	unsigned char buf[30];
	unsigned char *p;
	unsigned short chksum;

	p = buf;
	(void) read(0, (char *)p, 30);
	if ( *p!=0xc5 || *(p+1)!=0xd0 || *(p+2)!=0xd3 || *(p+3)!=0xc6 )
		error("Not a EPS file...");
	ps_offset = BYTE2LONG(p+4);
	ps_length = BYTE2LONG(p+8);
	mf_offset = BYTE2LONG(p+12);
	mf_length = BYTE2LONG(p+16);
	tiff_offset = BYTE2LONG(p+20);
	tiff_length = BYTE2LONG(p+24);
	checksum = BYTE2SHORT(p+28);
	if ( checksum != 0xffff ) {
		for(chksum = 0; p - buf < 28; p++ )
			chksum ^= *p;
		if ( chksum != checksum )
			error("Bad EPS header checksum...");
	}
	if ( tiff_offset == 0 || tiff_length == 0 )
		error("EPS Image not in TIFF format...");
}

gen_tiff_file()
{
	char *buf;

	fp = fopen(TIFF_TMP, "w");
	(void) lseek(0, tiff_offset, L_SET);
	buf = malloc(tiff_length+1);
	(void) read(0, buf, (int)tiff_length);
	(void) write(fileno(fp), buf, (int)tiff_length);
	free(buf);
	(void) fclose(fp);
}

gen_ps_file()
{
	char c;

	fp = fopen(PS_TMP, "w");
	(void) lseek(0, ps_offset, L_SET);
	while ( ps_length-- > 0 ) {
		c = getchar();
		if ( c == '\r' ) {
			(void) fputc('\n', fp);
			if ( (c = getchar()) != '\n')
				(void) fputc(c, fp);
			ps_length--;
		}
		else
			(void) fputc(c, fp);
	}
	(void) fclose(fp);
}

process_files()
{
	char buf[256];
	char *p; 

	fp = fopen(PS_TMP, "r");
	while ( fgets(buf, 256, fp) != NULL ) {
		(void) fputs(buf, stdout);
		p = strtok(buf, " \t");
		if ( STREQ(p, "%%EndComments\n") )
			process_tiff();
	}
	(void) fclose(fp);
}

process_tiff()
{

	TIFF *TIFFOpen(), *tp;
	char *buf;
	unsigned tiff_nbytes;
	int row;
	unsigned short config;
	
	tiff_photo = 0;		/* minimum is white by default */
	tp = TIFFOpen(TIFF_TMP, "r");
	TIFFGetField(tp, TIFFTAG_PLANARCONFIG, &config);
	tiff_nbytes = TIFFScanlineSize(tp);
        buf = malloc(tiff_nbytes);
	if ( buf == NULL ) 
		error("No space for scanline buffer...");
	TIFFGetField(tp, TIFFTAG_IMAGEWIDTH, &tiff_width);
	TIFFGetField(tp, TIFFTAG_IMAGELENGTH, &tiff_height);
	TIFFGetField(tp, TIFFTAG_PHOTOMETRIC, &tiff_photo);
	(void) printf("%%%%BeginPreview\n");
	(void) printf("%%%%ImageWidth: %d\n", tiff_width);
	(void) printf("%%%%ImageHeight: %d\n", tiff_height);
	(void) printf("%%%%ImageDepth: 1\n");
	(void) printf("%%%%BeginImage: %d\n", tiff_height);
	for (row = 0; row < tiff_height; row++) {
		if ( TIFFReadScanline(tp, buf, row, 0) < 0 )
			break;
		hexdump(buf, tiff_nbytes);
	}
	TIFFClose(tp);
	(void) printf("%%%%EndPreview\n");
	(void) printf("%%%%EndImage\n");
}

hexdump(pp, nbytes)
register char *pp;
unsigned nbytes;
{
	register unsigned i;
	char bits;

	(void) printf("%% ");
	for(i = 0; i < nbytes; i++) {
		bits = *pp++;
		if ( tiff_photo == 1 )
			bits = ~bits;
		(void) printf("%02x", bits & 0xff);
	}
	putchar('\n');
}

cleanup_files()
{
	(void) unlink(PS_TMP);
	(void) unlink(TIFF_TMP);
}

error(s)
char *s;
{
	(void) fprintf(stderr,"%s\n", s);
	exit (-1);
}
